@@ -33,12 +33,14 @@
 #include <gringo/scripts.hh>
 #include <gringo/version.hh>
 #include <gringo/control.hh>
+#include <climits>
 #include <iostream>
 #include <stdexcept>
 #include <program_opts/application.h>
 #include <program_opts/typed_value.h>
 
 struct GringoOptions {
+    using Foobar = std::vector<Gringo::FWSignature>;
 	ProgramOptions::StringSeq   defines;
     Gringo::Output::LparseDebug lparseDebug     = Gringo::Output::LparseDebug::NONE;
     bool                        verbose         = false;
@@ -50,8 +52,32 @@ struct GringoOptions {
     bool                        wNoAtomUndef    = false;
     bool                        wNoNonMonotone  = false;
     bool                        wNoFileIncluded = false;
+    Foobar foobar;
 };
 
+static inline std::vector<std::string> split(std::string const &source, char const *delimiter = " ", bool keepEmpty = false) {
+    std::vector<std::string> results;
+    size_t prev = 0;
+    size_t next = 0;
+    while ((next = source.find_first_of(delimiter, prev)) != std::string::npos) {
+        if (keepEmpty || (next - prev != 0)) { results.push_back(source.substr(prev, next - prev)); }
+        prev = next + 1;
+    }
+    if (prev < source.size()) { results.push_back(source.substr(prev)); }
+    return results;
+}
+
+static inline bool parseFoobar(const std::string& str, GringoOptions::Foobar& foobar) {
+    for (auto &x : split(str, ",")) {
+        auto y = split(x, "/");
+        if (y.size() != 2) { return false; }
+        unsigned a;
+        if (!bk_lib::string_cast<unsigned>(y[1], a)) { return false; }
+        foobar.emplace_back(y[0], a);
+    }
+    return true;
+}
+
 #define LOG if (opts.verbose) std::cerr
 struct IncrementalControl : Gringo::Control {
     using StringVec = std::vector<std::string>;
@@ -79,72 +105,96 @@ struct IncrementalControl : Gringo::Control {
             LOG << "reading from stdin" << std::endl;
             parser.pushFile("-");
         }
-        parser.parse();
-        LOG << "************** parsed program **************" << std::endl << prg;
-        LOG << "*********** extensional database ***********" << std::endl;
-        prg.rewrite(defs);
-        LOG << "************* rewritten program ************" << std::endl << prg;
-        prg.check();
-        if (message_printer()->hasError()) {
-            throw std::runtime_error("grounding stopped because of errors");
+        parse();
+    }
+    void parse() {
+        if (!parser.empty()) {
+            parser.parse();
+            defs.init();
+            parsed = true;
+        }
+    }
+    virtual void ground(Gringo::Control::GroundVec const &parts, Gringo::Any &&context) { 
+        // NOTE: it would be cool to have assumptions in the lparse output
+        auto exit = Gringo::onExit([this]{ scripts.context = Gringo::Any(); });
+        scripts.context = std::move(context);
+        parse();
+        if (parsed) {
+            LOG << "************** parsed program **************" << std::endl << prg;
+            prg.rewrite(defs);
+            LOG << "************* rewritten program ************" << std::endl << prg;
+            prg.check();
+            if (Gringo::message_printer()->hasError()) {
+                throw std::runtime_error("grounding stopped because of errors");
+            }
+            parsed = false;
+        }
+        if (!grounded) {
+            if (incremental) { out.incremental(); }
+            grounded = true;
+        }
+        if (!parts.empty()) {
+            Gringo::Ground::Parameters params;
+            for (auto &x : parts) { params.add(x.first, x.second); }
+            Gringo::Ground::Program gPrg(prg.toGround(out.domains));
+            LOG << "************* intermediate program *************" << std::endl << gPrg << std::endl;
+            LOG << "*************** grounded program ***************" << std::endl;
+            gPrg.ground(params, scripts, out, false);
         }
     }
-    
-    virtual void ground(std::string const &name, Gringo::FWValVec args) { params.add(name, args); }
     virtual void add(std::string const &name, Gringo::FWStringVec const &params, std::string const &part) {
         Gringo::Location loc("<block>", 1, 1, "<block>", 1, 1);
         Gringo::Input::IdVec idVec;
         for (auto &x : params) { idVec.emplace_back(loc, x); }
-        parts.emplace_back(name, std::move(idVec), part);
+        parser.pushBlock(name, std::move(idVec), part);
+        parse();
     }
     virtual Gringo::Value getConst(std::string const &name) {
+        parse();
         auto ret = defs.defs().find(name);
         if (ret != defs.defs().end()) {
             return std::get<2>(ret->second)->eval();
         }
         return Gringo::Value();
     }
+    virtual void load(std::string const &filename) {
+        parser.pushFile(std::string(filename));
+        parse();
+    }
     virtual void onModel(Gringo::Model const &) { }
     virtual bool blocked() { return false; }
-    virtual Gringo::SolveResult solve(ModelHandler) { 
-        if (!parts.empty()) {
-            parser.pushBlocks(std::move(parts));
-            parser.parse();
-            parts.clear();
-        }
-        Gringo::Ground::Program gPrg(prg.toGround(out.domains));
-        LOG << "************* intermediate program *************" << std::endl << gPrg << std::endl;
-        LOG << "*************** grounded program ***************" << std::endl;
-        gPrg.ground(params, scripts, out, false);
-        for (auto &ext : freeze) {
-            Gringo::PredicateDomain::element_type *atm = out.find2(ext.first);
-            if (atm->second.hasUid()) {
-                out.external(*atm, ext.second);
-            }
-        }
+    virtual Gringo::SolveResult solve(ModelHandler, Assumptions &&ass) {
+        if (!ass.empty()) { std::cerr << "warning: the lparse format does not support assumptions" << std::endl; }
         out.finish();
-        freeze.clear();
-        params.clear();
+        grounded = false;
         return Gringo::SolveResult::UNKNOWN;
     }
-    virtual Gringo::SolveFuture *asolve(ModelHandler, FinishHandler) { throw std::runtime_error("solving not supported in gringo"); }
+    virtual Gringo::SolveIter *solveIter(Assumptions &&) { 
+        throw std::runtime_error("solving not supported in gringo");
+    }
+    virtual Gringo::SolveFuture *solveAsync(ModelHandler, FinishHandler, Assumptions &&) { throw std::runtime_error("solving not supported in gringo"); }
     virtual Gringo::Statistics *getStats() { throw std::runtime_error("statistics not supported in gringo (yet)"); }
-    virtual void assignExternal(Gringo::Value ext, bool val) { freeze.emplace_back(ext, val ? Gringo::Output::ExternalType::E_TRUE : Gringo::Output::ExternalType::E_FALSE); }
-    virtual void releaseExternal(Gringo::Value ext)          { freeze.emplace_back(ext, Gringo::Output::ExternalType::E_FREE); }
-    virtual void setConf(std::string const &, bool) { }
-    virtual void enableEnumAssumption(bool) { }
+    virtual void assignExternal(Gringo::Value ext, Gringo::TruthValue val) { 
+        Gringo::PredicateDomain::element_type *atm = out.find2(ext);
+        if (atm && atm->second.hasUid()) {
+            out.external(*atm, val);
+        }
+    }
+    virtual Gringo::ConfigProxy &getConf() { throw std::runtime_error("configuration not supported"); }
+    virtual void useEnumAssumption(bool) { }
+    virtual bool useEnumAssumption() { return false; }
     virtual ~IncrementalControl() { }
 
-    std::vector<std::pair<Gringo::Value, Gringo::Output::ExternalType>> freeze;
     Gringo::Output::OutputBase            &out;
     Gringo::Scripts                        scripts;
     Gringo::Defines                        defs;
     Gringo::Input::Program                 prg;
     Gringo::Input::NongroundProgramBuilder pb;
     Gringo::Input::NonGroundParser         parser;
-    Gringo::Ground::Parameters             params;
-    Gringo::Input::ProgramVec              parts;
     GringoOptions const                   &opts;
+    bool                                   parsed = false;
+    bool                                   grounded = false;
+    bool                                   incremental = false;
 };
 #undef LOG
 
@@ -158,8 +208,8 @@ static bool parseWarning(const std::string& str, GringoOptions& out) {
     if (str ==    "atom-undefined")        { out.wNoAtomUndef    = false; return true; }
     if (str == "no-define-cyclic")         { out.wNoCycle        = true;  return true; }
     if (str ==    "define-cyclic")         { out.wNoCycle        = false; return true; }
-    if (str == "no-define-redfinition")    { out.wNoRedef        = true;  return true; }
-    if (str ==    "define-redfinition")    { out.wNoRedef        = false; return true; }
+    if (str == "no-define-redefinition")   { out.wNoRedef        = true;  return true; }
+    if (str ==    "define-redefinition")   { out.wNoRedef        = false; return true; }
     if (str == "no-file-included")         { out.wNoFileIncluded = true;  return true; }
     if (str ==    "file-included")         { out.wNoFileIncluded = false; return true; }
     if (str == "no-nonmonotone-aggregate") { out.wNoNonMonotone  = true;  return true; }
@@ -195,10 +245,11 @@ protected:
             ("warn,W"                   , storeTo(grOpts_, parseWarning)->arg("<warn>")->composing(), "Enable/disable warnings:\n"
              "      [no-]atom-undefined:        a :- b.\n"
              "      [no-]define-cyclic:         #const a=b. #const b=a.\n"
-             "      [no-]define-redfinition:    #const a=1. #const a=2.\n"
+             "      [no-]define-redefinition:   #const a=1. #const a=2.\n"
              "      [no-]file-included:         #include \"a.lp\". #include \"a.lp\".\n"
              "      [no-]nonmonotone-aggregate: a :- #sum { 1:a; -1:a } >= 0.\n"
              "      [no-]term-undefined       : p(1/0).\n")
+            ("foobar,@4"                , storeTo(grOpts_.foobar, parseFoobar)     , "Foobar")
             ;
         root.add(gringo);
         OptionGroup basic("Basic Options");
@@ -252,12 +303,14 @@ protected:
         using namespace Gringo;
         IncrementalControl inc(out, input_, grOpts_);
         if (inc.scripts.callable("main")) { 
-            out.incremental();
+            inc.incremental = true;
             inc.scripts.main(inc);
         }
         else { 
-            inc.params.add("base", {});
-            inc.solve(nullptr);
+            Gringo::Control::GroundVec parts;
+            parts.emplace_back("base", FWValVec{});
+            inc.ground(parts, Gringo::Any());
+            inc.solve(nullptr, {});
         }
     }
 
@@ -265,6 +318,9 @@ protected:
         using namespace Gringo;
         grOpts_.verbose = verbose() == UINT_MAX;
         Output::OutputPredicates outPreds;
+        for (auto &x : grOpts_.foobar) {
+            outPreds.emplace_back(Location("<cmd>",1,1,"<cmd>", 1,1), x, false);
+        }
         if (grOpts_.text) {
             Output::OutputBase out(std::move(outPreds), std::cout, grOpts_.lpRewrite);
             ground(out);
