@@ -30,27 +30,6 @@
 #include <clasp/util/misc_types.h>
 #include <map>
 
-#define CLASP_ASP_OPTIONS(OPTION, SELF) \
-OPTION("supp-models"  , supp_models, X(flag())    , "Compute supported models (no unfounded set check)", SET_LEQ(SELF.suppMod, 1u)) \
-OPTION("eq,@1"        , eq         , X(arg("<n>")), "Configure equivalence preprocessing\n" \
-	"      Run for at most %A iterations (-1=run to fixpoint)", SET_OR_FILL(SELF.iters)) \
-OPTION("backprop!,@1" , backprop   , X(flag())    , "Use backpropagation in ASP-preprocessing", SET_LEQ(SELF.backprop,1u)) \
-OPTION("no-gamma,@1"  , no_gamma   , X(flag())    , "Do not add gamma rules for non-hcf disjunctions", SET_LEQ(SELF.noGamma,1u)) \
-OPTION("eq-dfs,@2"    , eq_dfs     , X(flag())    , "Enable df-order in eq-preprocessing", SET_LEQ(SELF.dfOrder,1u)) \
-OPTION("trans-ext!,@1", trans_ext  , _0           , "Configure handling of Lparse-like extended rules\n" \
-	"      %A: {all|choice|card|weight|integ|dynamic}\n" \
-	"        all    : Transform all extended rules to basic rules\n" \
-	"        choice : Transform choice rules, but keep cardinality and weight rules\n" \
-	"        card   : Transform cardinality rules, but keep choice and weight rules\n" \
-	"        weight : Transform cardinality and weight rules, but keep choice rules\n" \
-	"        scc    : Transform \"recursive\" cardinality and weight rules\n" \
-	"        integ  : Transform cardinality integrity constraints\n" \
-	"        dynamic: Transform \"simple\" extended rules, but keep more complex ones", STORE_ENUM(SELF.erMode,    \
-	_2("no"    , Asp::LogicProgram::mode_native)          , _2("all" , Asp::LogicProgram::mode_transform),    \
-	_2("choice", Asp::LogicProgram::mode_transform_choice), _2("card", Asp::LogicProgram::mode_transform_card),\
-	_2("weight", Asp::LogicProgram::mode_transform_weight), _2("scc" , Asp::LogicProgram::mode_transform_scc),\
-	_2("integ" , Asp::LogicProgram::mode_transform_integ) , _2("dynamic", Asp::LogicProgram::mode_transform_dynamic)))
-
 namespace Clasp { namespace Asp {
 //! Program statistics for *one* incremental step.
 class LpStats {
@@ -114,6 +93,7 @@ public:
 
 	//! Options for the Asp-Preprocessor.
 	struct AspOptions {
+		static const uint32 MAX_EQ_ITERS = static_cast<uint32>( (1u<<26)-1 );
 		AspOptions() : erMode(mode_native), iters(5), noSCC(0), dfOrder(0), backprop(0), normalize(0), suppMod(0), noGamma(0) {}
 		AspOptions& iterations(uint32 it)   { iters   = it;return *this;}
 		AspOptions& depthFirst()            { dfOrder = 1; return *this;}
@@ -249,8 +229,12 @@ public:
 	/*!
 	 * If the atom is defined in this or a previous step, the operation has no effect. 
 	 * \note If atomId is not yet known, an atom with the given id is implicitly created.
-	 * \note The second parameter defines the value that is assumed for the given atom while
-	 *       it is frozen. It shall be either value_false or value_true.
+	 * \note The second parameter defines the assumption that shall hold during solving, i.e.
+	 *       - True(atomId) , if value is value_true,
+	 *       - False(atomId), if value is value_false, or
+	 *       - no assumption, if value is value_free.
+	 *
+	 * \see ProgramBuilder::getAssumptions(LitVec&) const;
 	 */
 	LogicProgram& freeze(Var atomId, ValueRep value = value_false);
 
@@ -393,7 +377,7 @@ public:
 	typedef std::pair<EdgeIterator, EdgeIterator> EdgeRange;
 	typedef std::pair<HeadIter, HeadIter>         HeadRange;
 	const AspOptions& options()    const { return opts_; }
-	bool       hasConflict()       const { return atoms_[0]->literal() == posLit(0); }
+	bool       hasConflict()       const { return getFalseAtom()->literal() == posLit(0); }
 	bool       ok()                const { return !hasConflict() && ProgramBuilder::ok(); }
 	PrgAtom*   getAtom(Var atomId) const { return atoms_[atomId]; }
 	PrgHead*   getHead(PrgEdge it) const { return it.isAtom() ? (PrgHead*)getAtom(it.node()) : (PrgHead*)getDisj(it.node()); }
@@ -419,7 +403,7 @@ public:
 	PrgAtom*   mergeEqAtoms(PrgAtom* a, Var rootAtom);
 	PrgBody*   mergeEqBodies(PrgBody* b, Var rootBody, bool hashEq, bool atomsAssigned);
 	bool       propagate()               { return propagate(options().backprop); }
-	void       setConflict()             { atoms_[0]->setLiteral(posLit(0)); }
+	void       setConflict()             { getFalseAtom()->setLiteral(posLit(0)); }
 	RuleState& ruleState()               { return ruleState_; }
 	void       addMinimize();
 	// ------------------------------------------------------------------------
@@ -434,9 +418,8 @@ private:
 	LogicProgram(const LogicProgram&);
 	LogicProgram& operator=(const LogicProgram&);	
 	typedef PodVector<Rule*>::type RuleList;
-	typedef std::multimap<uint32, uint32>   BodyIndex; // hash -> bodies[offset]
-	typedef std::multimap<uint32, uint32>   DisjIndex; // hash -> bodies[offset]
-	typedef BodyIndex::iterator             IndexIter;
+	typedef std::multimap<uint32, uint32>   IndexMap; // hash -> vec[offset]
+	typedef IndexMap::iterator              IndexIter;
 	typedef std::pair<IndexIter, IndexIter> IndexRange;
 	typedef PodVector<uint8>::type          SccMap;
 	struct MinimizeRule;
@@ -450,12 +433,14 @@ private:
 	int  doType() const { return Problem_t::ASP; }
 	// ------------------------------------------------------------------------
 	// Program definition
+	bool     isNew(Var atomId) const { return atomId >= startAtom(); }
 	PrgAtom* resize(Var atomId);
+	PrgAtom* setExternal(Var atomId, ValueRep v);
 	void     addRuleImpl(RuleType t,    const VarVec& head, BodyInfo& body);
 	bool     handleNatively(RuleType t, const BodyInfo& i) const;
 	bool     transformNoAux(RuleType t, const BodyInfo& i) const;
 	void     transformExtended();
-	void     transformIntegrity(uint32 maxAux);
+	void     transformIntegrity(uint32 nAtoms, uint32 maxAux);
 	PrgBody* getBodyFor(BodyInfo& body, bool addDeps = true);
 	PrgDisj* getDisjFor(const VarVec& heads, uint32 headHash);
 	PrgBody* assignBodyFor(BodyInfo& body, EdgeType x, bool strongSimp);
@@ -474,8 +459,9 @@ private:
 		NodeType n = vec[id];
 		NodeType r;
 		Var root   = n->id();
-		for (r = vec[root]; r->eq(); r = vec[root]) {
-			// if n == r and r == r' -> n == r'
+		for (r = vec[root]; r->eq(); r = vec[root]) { 
+			// n == r and r == r' -> n == r'
+			assert(root != r->id());
 			n->setEq(root = r->id());
 		}
 		return root;
@@ -488,17 +474,18 @@ private:
 	void simplifyMinimize();
 	bool addConstraints();
 	// ------------------------------------------------------------------------
-	void writeBody(const BodyInfo& body, std::ostream&) const;
-	void transform(const PrgBody& body, BodyInfo& out)  const;
+	bool transform(const PrgBody& body, BodyInfo& out)  const;
 	void transform(const MinimizeRule&, BodyInfo& out)  const;
-	Var  getFalseAtom() const;
+	Var      getFalseId()      const { return 0; }
+	PrgAtom* getFalseAtom()    const { return atoms_[0]; }
+	Var      findLpFalseAtom() const;
 	Rule           rule_;        // active rule
 	BodyInfo       activeBody_;  // simplified body of active rule
 	VarVec         activeHead_;  // simplified head of active rule
 	RuleState      ruleState_;   // which atoms appear in the active rule?
 	RuleList       extended_;    // extended rules to be translated
-	BodyIndex      bodyIndex_;   // hash -> body
-	DisjIndex      disjIndex_;   // hash -> disjunction
+	IndexMap       bodyIndex_;   // hash -> body
+	IndexMap       disjIndex_;   // hash -> disjunction
 	BodyList       bodies_;      // all bodies
 	AtomList       atoms_;       // all atoms
 	AtomList       sccAtoms_;    // atoms that are strongly connected
@@ -514,12 +501,12 @@ private:
 	struct Incremental  {
 		Incremental();
 		uint32  startAtom;// first atom of current iteration
-		uint32  startAux; // first aux atom of current iteration
 		uint32  startScc; // first valid scc number in this iteration
 		VarVec  frozen;   // list of frozen atoms
 		VarVec  update;   // list of atoms to be updated (freeze/unfreeze) in this step
-	}*             incData_;// additional state to handle incrementally defined programs 
-	AspOptions     opts_;   // preprocessing 
+	}*             incData_;     // additional state to handle incrementally defined programs 
+	AspOptions     opts_;        // preprocessing 
+	uint32         startAux_;    // first aux atom
 };
 } } // end namespace Asp
 #endif
