@@ -28,6 +28,16 @@
 #include <climits>
 #include <string>
 #include <cstdlib>
+#if defined(__cplusplus) && __cplusplus >= 201103L
+#include <cmath>
+#else 
+#ifdef _MSC_VER
+#include <float.h>
+#define isnan _isnan
+#else
+#include <math.h>
+#endif
+#endif
 #if !defined(_WIN32)
 #include <signal.h>
 #elif !defined(SIGALRM)
@@ -107,9 +117,10 @@ void Output::setCallQuiet(PrintLevel call)  { quiet_[2] = static_cast<uint8>(cal
 void Output::setHide(char c)                { hidePref_ = c; }
 
 void Output::saveModel(const Model& m) {
-	vals_         = *m.values;
-	saved_        = m;
-	saved_.values = &vals_;
+	Model temp(m);
+	temp.values = &(vals_ = *m.values);
+	temp.costs  = m.costs ? &(costs_=*m.costs) : 0;
+	saved_      = temp;
 }
 
 void Output::onEvent(const Event& ev) {
@@ -254,9 +265,9 @@ void JsonOutput::stopStep(const ClaspFacade::Summary& s) {
 	while (popObject() != '{') { ; }
 }
 
-void JsonOutput::visitCoreSolverStats(double cpuTime, uint64 models, const SolverStats& st, bool) {
+void JsonOutput::visitCoreSolverStats(double cpuTime, uint64 models, const SolverStats& st, bool accu) {
 	pushObject("Core");
-	printKeyValue("CPU Time"   , cpuTime);
+	if (!accu) { printKeyValue("CPU", cpuTime); }
 	printKeyValue("Models"     , models);
 	printKeyValue("Choices"    , st.choices);
 	printKeyValue("Conflicts"  , st.conflicts);
@@ -434,7 +445,11 @@ void JsonOutput::printKeyValue(const char* k, uint64 v) {
 }
 void JsonOutput::printKeyValue(const char* k, uint32 v) { return printKeyValue(k, uint64(v)); }
 void JsonOutput::printKeyValue(const char* k, double v) {
-	printf("%s%-*s\"%s\": %.3f", open_, indent(), " ", k, v);
+#if defined(__cplusplus) && __cplusplus >= 201103L
+    using std::isnan;
+#endif
+	if (!isnan(v)) { printf("%s%-*s\"%s\": %.3f", open_, indent(), " ", k, v); } 
+	else           { printf("%s%-*s\"%s\": %s", open_, indent(), " ", k, "null"); }
 	open_ = ",\n";
 }
 
@@ -495,12 +510,12 @@ void JsonOutput::printModel(const SymbolTable& index, const Model& m, PrintLevel
 	}
 	if (hasModel) { popObject(); }
 }
-void JsonOutput::printCosts(const SharedMinimizeData& opt) { 
+void JsonOutput::printCosts(const SumVec& opt) { 
 	pushObject("Costs", type_array);
 	printf("%-*s", indent(), " ");
 	const char* sep = "";
-	for (uint32 i = 0, end = opt.numRules(); i != end; ++i) {
-		printf("%s%" PRId64, sep, opt.optimum(i));
+	for (SumVec::const_iterator it = opt.begin(), end = opt.end(); it != end; ++it) {
+		printf("%s%" PRId64, sep, *it);
 		sep = ", ";
 	}
 	popObject();
@@ -756,8 +771,9 @@ void TextOutput::printSolveProgress(const Event& ev) {
 	fflush(stdout);
 }
 
+const char* TextOutput::fieldSeparator() const { return ifs_; }
 int TextOutput::printSep(CategoryKey k) const {
-	return printf("%s%s", ifs_, *ifs_ != '\n' ? "" : format[k]);
+	return printf("%s%s", fieldSeparator(), *fieldSeparator() != '\n' ? "" : format[k]);
 }
 void TextOutput::printModel(const SymbolTable& sym, const Model& m, PrintLevel x) {
 	if (x == modelQ()) {
@@ -765,13 +781,14 @@ void TextOutput::printModel(const SymbolTable& sym, const Model& m, PrintLevel x
 		printf("%s", format[cat_value]);
 		if (sym.type() == SymbolTable::map_indirect) { printNames(sym, m); }
 		else {
-			uint32 const maxLineLen = *ifs_ == ' ' ? 70 : UINT32_MAX;
+			uint32 const maxLineLen = *fieldSeparator() == ' ' ? 70 : UINT32_MAX;
 			uint32       printed    = 0;
 			bool   const onlyTrue   = m.consequences();
+			std::string  fmt("%s"); fmt += format[cat_atom];
 			for (Var v = 1; v < sym.size(); ++v) {
 				if (!onlyTrue || m.isTrue(posLit(v))) {
 					if (printed) { printed += printSep(cat_value); }
-					printed += printf(format[cat_atom], m.value(v) == value_false ? -int(v) : int(v));
+					printed += printf(fmt.c_str(), m.value(v) == value_false ? "-":"", int(v));
 					if (printed >= maxLineLen) {
 						printed = 0;
 						printf("\n%s", format[cat_value]);
@@ -804,11 +821,11 @@ void TextOutput::printNames(const Clasp::SymbolTable& sym, const Clasp::Model& m
 	}
 }
 
-void TextOutput::printCosts(const SharedMinimizeData& costs) const {
-	printf("%" PRId64, costs.optimum(0));
-	for (uint32 i = 1, end = costs.numRules(); i != end; ++i) {
+void TextOutput::printCosts(const SumVec& costs) const {
+	printf("%" PRId64, costs[0]);
+	for (uint32 i = 1, end = (uint32)costs.size(); i != end; ++i) {
 		printSep(cat_objective);
-		printf("%" PRId64, costs.optimum(i));
+		printf("%" PRId64, costs[i]);
 	}
 }
 void TextOutput::startSection(const char* n) const {
@@ -898,21 +915,18 @@ void TextOutput::visitExtSolverStats(const ExtendedStats& stx, bool accu) {
 	}
 	printKeyValue("Problems", "%-8" PRIu64,  (uint64)stx.gps);
 	printf(" (Average Length: %.2f Splits: %" PRIu64")\n", stx.avgGp(), (uint64)stx.splits);
-	uint64 sum          = stx.lemmas();
+	uint64 sum = stx.lemmas();
 	printKeyValue("Lemmas", "%-8" PRIu64, sum);
 	printf(" (Deleted: %" PRIu64")\n",  stx.deleted);
+	printKeyValue("  Binary", "%-8" PRIu64, uint64(stx.binary));
+	printf(" (Ratio: %6.2f%%)\n", percent(stx.binary, sum));
+	printKeyValue("  Ternary", "%-8" PRIu64, uint64(stx.ternary));
+	printf(" (Ratio: %6.2f%%)\n", percent(stx.ternary, sum));
 	const char* names[] = {"  Conflict", "  Loop", "  Other"};
-	for (int i = 0; i <= Constraint_t::learnt_other; ++i) {
-		if (i == 0) {
-			printKeyValue("  Binary", "%-8" PRIu64, uint64(stx.binary));
-			printf(" (Ratio: %6.2f%%)\n", percent(stx.binary, sum));
-			printKeyValue("  Ternary", "%-8" PRIu64, uint64(stx.ternary));
-			printf(" (Ratio: %6.2f%%)\n", percent(stx.ternary, sum));
-		}
-		else {
-			printKeyValue(names[i-1], "%-8" PRIu64, stx.lemmas(ConstraintType(i)));
-			printf(" (Average Length: %6.1f Ratio: %6.2f%%) \n", stx.avgLen(ConstraintType(i)), percent(stx.lemmas(ConstraintType(i)), sum));
-		}
+	for (int i = 0; i != sizeof(names)/sizeof(names[0]); ++i) {
+		ConstraintType type = static_cast<ConstraintType>(i+1);
+		printKeyValue(names[i], "%-8" PRIu64, stx.lemmas(type));
+		printf(" (Average Length: %6.1f Ratio: %6.2f%%) \n", stx.avgLen(type), percent(stx.lemmas(type), sum));
 	}
 	if (stx.distributed || stx.integrated) {
 		printKeyValue("  Distributed", "%-8" PRIu64, stx.distributed);
