@@ -20,6 +20,8 @@
 #include <clasp/minimize_constraint.h>
 #include <clasp/solver.h>
 #include <clasp/weight_constraint.h>
+#include <clasp/clause.h>
+#include <stdio.h>
 namespace Clasp { 
 /////////////////////////////////////////////////////////////////////////////////////////
 // SharedMinimizeData
@@ -41,14 +43,26 @@ void SharedMinimizeData::resetBounds() {
 	gCount_  = 0;
 	optGen_  = 0;
 	lower_.assign(numRules(), 0);
-	opt_[0].assign(numRules(), maxBound());
-	opt_[1] = opt_[0];
+	up_[0].assign(numRules(), maxBound());
+	up_[1].assign(numRules(), maxBound());
+	const WeightLiteral* lit = lits;
+	for (weight_t wPos = 0, end = (weight_t)weights.size(), x; wPos != end; wPos = x+1) {
+		assert(weights[wPos].weight >= 0);
+		for (x = wPos; weights[x].next; ) { // compound weight - check for negative
+			if (weights[++x].weight < 0) {
+				while (lit->second != wPos) { ++lit; }
+				for (const WeightLiteral* t = lit; t->second == wPos; ++t) {
+					lower_[weights[x].level] += weights[x].weight;
+				}
+			}
+		}
+	}
 }
 
 bool SharedMinimizeData::setMode(MinimizeMode m, const wsum_t* bound, uint32 boundSize)  {
 	mode_ = m;
 	if (boundSize && bound) {
-		SumVec& opt = opt_[0];
+		SumVec& opt = up_[0];
 		bool    ok  = false;
 		gCount_     = 0;
 		optGen_     = 0;
@@ -66,40 +80,31 @@ bool SharedMinimizeData::setMode(MinimizeMode m, const wsum_t* bound, uint32 bou
 	return true;
 }
 
-MinimizeConstraint* SharedMinimizeData::attach(Solver& s, uint32 strat, bool addRef) {
+MinimizeConstraint* SharedMinimizeData::attach(Solver& s, MinimizeMode_t::Strategy strat, uint32 param, bool addRef) {
 	if (addRef) this->share();
-	const SolverParams& c = s.configuration();
-	if (strat == UINT32_MAX) {
-		strat = c.optStrat;
-	}
-	if ((c.optHeu & SolverStrategies::opt_sign) != 0) {
-		for (const WeightLiteral* it = lits; !isSentinel(it->first); ++it) {
-			s.setPref(it->first.var(), ValueSet::pref_value, falseValue(it->first));
-		}
-	}
 	MinimizeConstraint* ret;
-	if (strat < SolverStrategies::opt_unsat || mode() == MinimizeMode_t::enumerate) {
-		ret = new DefaultMinimize(this, strat);
+	if (strat == MinimizeMode_t::opt_bb || mode() == MinimizeMode_t::enumerate) {
+		ret = new DefaultMinimize(this, param);
 	}
 	else {
-		ret = new UncoreMinimize(this, strat == SolverStrategies::opt_unsat_pre);
+		ret = new UncoreMinimize(this, param);
 	}
 	ret->attach(s);
 	return ret;
 }
 
 const SumVec* SharedMinimizeData::setOptimum(const wsum_t* newOpt) {
-	if (mode() == MinimizeMode_t::enumerate) {
-		opt_[1].assign(newOpt, newOpt+numRules());
-		return opt_ + 1;
+	if (optGen_) { return up_ + (optGen_&1u); }
+	uint32  g = gCount_;
+	uint32  n = 1u - (g & 1u);
+	SumVec& U = up_[n];
+	U.assign(newOpt, newOpt + numRules());
+	assert(mode() != MinimizeMode_t::enumerate || n == 1);
+	if (mode() != MinimizeMode_t::enumerate) {
+		if (++g == 0) { g = 2; }
+		gCount_  = g;
 	}
-	if (optGen_) { return opt_ + (optGen_&1u); }
-	uint32 g = gCount_;
-	uint32 n = 1u - (g & 1u);
-	opt_[n].assign(newOpt, newOpt+numRules());
-	if (++g == 0) { g = 2; }
-	gCount_  = g;
-	return opt_ + n;
+	return &U;
 }
 void SharedMinimizeData::setLower(uint32 lev, wsum_t low) {
 	lower_[lev] = low;
@@ -107,18 +112,6 @@ void SharedMinimizeData::setLower(uint32 lev, wsum_t low) {
 void SharedMinimizeData::markOptimal() {
 	optGen_ = generation();
 }
-bool SharedMinimizeData::heuristic(Solver& s, bool full) const {
-	const bool heuristic = full || (s.queueSize() == 0 && s.decisionLevel() == s.rootLevel());
-	if (heuristic && s.propagate()) {
-		for (const WeightLiteral* w = lits; !isSentinel(w->first); ++w) {
-			if (s.value(w->first.var()) == value_free) {
-				s.assume(~w->first);
-				if (!full || !s.propagate()) { break; }
-			}
-		}
-	}
-	return !s.hasConflict();
-}
 void SharedMinimizeData::sub(wsum_t* lhs, const LevelWeight* w, uint32& aLev) const {
 	if (w->level < aLev) { aLev = w->level; }
 	do { lhs[w->level] -= w->weight; } while (w++->next);
@@ -152,9 +145,6 @@ void MinimizeConstraint::destroy(Solver* s, bool d) {
 	shared_ = 0;
 	Constraint::destroy(s, d);
 }
-Constraint* MinimizeConstraint::cloneAttach(Solver& s) {
-	return shared_->attach(s);
-}
 /////////////////////////////////////////////////////////////////////////////////////////
 // DefaultMinimize
 /////////////////////////////////////////////////////////////////////////////////////////
@@ -176,9 +166,9 @@ DefaultMinimize::DefaultMinimize(SharedData* d, uint32 strat)
 	, undo_(0)
 	, undoTop_(0)
 	, size_(d->numRules()) {
-	step_.type = strat;
-	if (step_.type == SolverStrategies::opt_hier && d->numRules() == 1) {
-		step_.type = SolverStrategies::opt_def;
+	step_.type = strat & 3u;
+	if (step_.type == MinimizeMode_t::bb_step_hier && d->numRules() == 1) {
+		step_.type = 0;
 	}
 }
 
@@ -333,7 +323,7 @@ bool DefaultMinimize::propagateImpl(Solver& s, PropMode m) {
 		assert(active());
 		// force implied literals
 		if (!s.isFalse(it->first) || (impLevel < DL && s.level(it->first.var()) > impLevel)) {
-			if (impLevel != DL) { DL = s.undoUntil(impLevel, true); }
+			if (impLevel != DL) { DL = s.undoUntil(impLevel, Solver::undo_pop_bt_level); }
 			ok = s.force(~it->first, impLevel, this, undoPos);
 		}
 	}
@@ -396,7 +386,7 @@ bool DefaultMinimize::minimize(Solver& s, Literal p, CCMinRecursive* rec) {
 // Stores the current sum as the shared optimum.
 void DefaultMinimize::commitUpperBound(const Solver&)  {
 	shared_->setOptimum(sum());
-	if (step_.type == SolverStrategies::opt_inc) { step_.size *= 2; }
+	if (step_.type == MinimizeMode_t::bb_step_inc) { step_.size *= 2; }
 }
 bool DefaultMinimize::commitLowerBound(const Solver&, bool upShared) {
 	bool act   = active() && shared_->checkNext();
@@ -405,8 +395,8 @@ bool DefaultMinimize::commitLowerBound(const Solver&, bool upShared) {
 		uint32 x = step_.lev;
 		wsum_t L = opt()[x] + 1;
 		stepLow()= L;
-		while (upShared && shared_->lower(x) < L)   { shared_->setLower(x, L); }
-		if (step_.type == SolverStrategies::opt_inc){ step_.size = 1; }
+		while (upShared && shared_->lower(x) < L) { shared_->setLower(x, L); }
+		if (step_.type == MinimizeMode_t::bb_step_inc){ step_.size = 1; }
 	}
 	return more;
 }
@@ -431,8 +421,8 @@ bool DefaultMinimize::relaxBound(bool full) {
 }
 
 void DefaultMinimize::stepInit(uint32 n) {
-	step_.size = uint32(step_.type != SolverStrategies::opt_dec);
-	if (step_.type) { step_.lev = n; if (n != size_) stepLow() = 0; }
+	step_.size = uint32(step_.type != MinimizeMode_t::bb_step_dec);
+	if (step_.type) { step_.lev = n; if (n != size_) stepLow() = 0-shared_->maxBound(); }
 	else            { step_.lev = shared_->maxLevel(); }
 }
 
@@ -451,8 +441,8 @@ bool DefaultMinimize::integrateBound(Solver& s) {
 		uint32 dl= s.decisionLevel() + 1;
 		if (!STRATEGY(imp(sum(), min, opt(), actLev_)) || (dl = computeImplicationSet(s, min, x)) > s.rootLevel()) {
 			for (--dl; !s.hasConflict() || s.resolveConflict(); ) {
-				if      (s.undoUntil(dl, true) > dl)         { s.backtrack(); }
-				else if (propagateImpl(s, propagate_new_opt)){ return true;   }
+				if      (s.undoUntil(dl, Solver::undo_pop_bt_level) > dl){ s.backtrack(); }
+				else if (propagateImpl(s, propagate_new_opt))            { return true;   }
 			}
 		}
 		if (!shared_->checkNext()) {
@@ -511,10 +501,10 @@ bool DefaultMinimize::updateBounds(bool applyStep) {
 				assert(U > L && B > L);
 				wsum_t diff = U - L;
 				uint32 half = static_cast<uint32>( (diff>>1) | (diff&1) );
-				if (step_.type == SolverStrategies::opt_inc) {
+				if (step_.type == MinimizeMode_t::bb_step_inc) {
 					step_.size = std::min(step_.size, half);
 				}
-				else if (step_.type == SolverStrategies::opt_dec) {
+				else if (step_.type == MinimizeMode_t::bb_step_dec) {
 					if (!step_.size) { step_.size = static_cast<uint32>(diff); }
 					else             { step_.size = half; }
 				}
@@ -684,6 +674,7 @@ MinimizeBuilder::SharedData* MinimizeBuilder::build(SharedContext& ctx) {
 			}
 		}
 	}
+	srep->resetBounds();
 	ready_ = true;
 	return srep;
 }
@@ -790,14 +781,15 @@ bool MinimizeBuilder::eqWeight(const SharedData::LevelWeight* lhs, const Weight&
 /////////////////////////////////////////////////////////////////////////////////////////
 // UncoreMinimize
 /////////////////////////////////////////////////////////////////////////////////////////
-UncoreMinimize::UncoreMinimize(SharedMinimizeData* d, bool pre) 
+UncoreMinimize::UncoreMinimize(SharedMinimizeData* d, uint32 strat) 
 	: MinimizeConstraint(d)
 	, enum_(0)
 	, sum_(new wsum_t[d->numRules()])
 	, auxInit_(UINT32_MAX)
 	, auxAdd_(0)
-	, freeOpen_(0) {
-	hasPre_ = static_cast<uint32>(pre);
+	, freeOpen_(0)
+	, options_(0) {
+	options_ = strat & 7u;
 }
 void UncoreMinimize::init() {
 	releaseLits();
@@ -883,38 +875,43 @@ bool UncoreMinimize::integrate(Solver& s) {
 
 // Initializes the next optimization level to look at.
 bool UncoreMinimize::initLevel(Solver& s) {
-	assert(shared_->optimize() || !next_);
 	initRoot(s);
 	sat_   = 0;
 	pre_   = 0;
 	path_  = 1;
 	init_  = 0;
 	sum_[0]= -1;
+	if (!fixLevel(s)) {
+		return false;
+	}
 	for (LitVec::const_iterator it = fix_.begin(), end = fix_.end(); it != end; ++it) {
 		if (!s.force(*it, eRoot_, this)) { return false; }
 	}
-	if (!shared_->optimize() || !assume_.empty()) {
-		return !s.hasConflict(); 
+	if (!shared_->optimize()) {
+		next_  = 0;
+		level_ = shared_->maxLevel();
+		lower_ = shared_->lower(level_);
+		upper_ = shared_->upper(level_);
+		return true;
 	}
-	releaseLits();
 	bool hasWeights = false;
 	for (uint32 level = (level_ + next_), n = 1; level <= shared_->maxLevel() && assume_.empty(); ++level) {
 		level_ = level;
-		lower_ = std::min(shared_->lower(level), wsum_t(0));
+		lower_ = 0;
 		upper_ = shared_->upper(level_);
 		for (const WeightLiteral* it = shared_->lits; !isSentinel(it->first); ++it) {
 			if (weight_t w = shared_->weight(*it, level_)){
 				Literal x = it->first;
+				if (w < 0) {
+					lower_ += w;
+					w       = -w;
+					x       = ~x;
+				}
 				if (s.value(x.var()) == value_free || s.level(x.var()) > eRoot_) {
-					if (w < 0) { 
-						w       = -w;
-						x       = ~x;
-						lower_ -= w;
-					}
 					addLit(x, w);
 					if (w != 1) { hasWeights = true; }
 				}
-				else if (s.isTrue(it->first)) {
+				else if (s.isTrue(x)) {
 					lower_ += w;
 				}
 			}
@@ -934,7 +931,7 @@ bool UncoreMinimize::initLevel(Solver& s) {
 			}
 		}
 	}
-	pre_  = hasPre_;
+	pre_  = (options_ & MinimizeMode_t::usc_preprocess) != 0u;
 	valid_= (pre_ == 0 && !hasWeights);
 	if (next_ && !s.hasConflict()) {
 		s.force(~tag_, Antecedent(0));
@@ -954,35 +951,39 @@ UncoreMinimize::LitData& UncoreMinimize::addLit(Literal p, weight_t w) {
 // Pushes the active assumptions from the active optimization level to
 // the root path.
 bool UncoreMinimize::pushPath(Solver& s) {
-	bool ok = !s.hasConflict() && !sat_ && (!path_ || (!next_ && !init_) || initLevel(s));
-	if (path_) {
-		typedef LitSet::iterator iterator;
-		iterator j = assume_.begin();
+	bool ok   = !s.hasConflict() && !sat_ && (!path_ || (!next_ && !init_) || initLevel(s));
+	bool path = path_ != 0;
+	while (path) {
+		uint32 j = 0;
+		path_    = uint32(path = false);
+		ok       = ok && s.simplify();
 		initRoot(s);
-		ok = ok && s.simplify();
-		for (LitSet::iterator it = j, end = assume_.end(); it != end; ++it) {
-			LitData& x = getData(it->id);
+		for (uint32 i = 0, end = assume_.size(), dl; i != end; ++i) {
+			LitData& x = getData(assume_[i].id);
 			if (x.assume) {
-				*j++ = *it;
-				if      (!ok || s.isTrue(it->lit)) { continue; }
-				else if (s.value(it->lit.var()) == value_free) {
-					ok    = s.pushRoot(it->lit);
+				assume_[j++] = assume_[i];
+				Literal lit  = assume_[i].lit;
+				if      (!ok || s.isTrue(lit)) { continue; }
+				else if (s.value(lit.var()) == value_free) {
+					ok    = path || s.pushRoot(lit);
 					aTop_ = s.rootLevel();
 				}
-				else if (s.level(it->lit.var()) > eRoot_ || hasCore(x)) {
-					todo_.push_back(LitPair(~it->lit, it->id)); 
-					ok   = s.force(it->lit, Antecedent(0));
+				else if (s.level(lit.var()) > eRoot_) {
+					todo_.push_back(LitPair(~lit, assume_[i].id));
+					ok = s.force(lit, Antecedent(0));
 				}
 				else {
-					lower_  += x.weight; 
-					x.weight = 0; 
-					x.assume = 0;
+					LitPair core(~lit, assume_[i].id);
+					dl   = s.decisionLevel();
+					ok   = addCore(s, &core, 1, x.weight);
+					end  = assume_.size();
+					path = path || (ok && s.decisionLevel() != dl);
 					--j;
 				}
 			}
 		}
-		assume_.erase(j, assume_.end());
-		path_ = 0;
+		shrinkVecTo(assume_, j);
+		CLASP_FAIL_IF(!sat_ && s.decisionLevel() != s.rootLevel(), "pushPath must be called on root level (%u:%u)", s.rootLevel(), s.decisionLevel()); 
 	}
 	if (sat_ || (ok && !validLowerBound())) {
 		ok   = false;
@@ -1015,10 +1016,13 @@ bool UncoreMinimize::relax(Solver& s, bool reset) {
 		LitVec ignore;
 		handleUnsat(s, false, ignore);
 	}
-	if ((reset && shared_->optimize()) || !assume_.empty() || level_ != shared_->maxLevel() || s.sharedContext()->concurrency() > 1) {
+	if ((reset && shared_->optimize()) || !assume_.empty() || level_ != shared_->maxLevel() || next_) {
 		detach(&s, true);
 		init();
 	}
+	else {
+		releaseLits();
+	}
 	if (!shared_->optimize()) {
 		gen_  = shared_->generation();
 		next_ = 0;
@@ -1039,6 +1043,13 @@ wsum_t* UncoreMinimize::computeSum(Solver& s) const {
 	return sum_;
 }
 
+void UncoreMinimize::setLower(wsum_t x) {
+	if (!pre_ && x > lower_) { 
+		fprintf(stderr, "*** WARNING: Fixing lower bound (%u - %u)\n", (uint32)lower_, (uint32)x); 
+		lower_ = x;
+	}
+}
+
 // Checks whether the current assignment in s is valid w.r.t the
 // bound stored in the shared data object.
 bool UncoreMinimize::valid(Solver& s) {
@@ -1089,7 +1100,7 @@ bool UncoreMinimize::handleUnsat(Solver& s, bool up, LitVec& out) {
 		if (sat_ == 0) {
 			if (s.hasStopConflict()) { return false; }
 			conflict_ = s.conflict();
-			if (s.strategy.search == SolverStrategies::no_learning) {
+			if (s.searchMode() == SolverStrategies::no_learning) {
 				conflict_.clear();
 				for (uint32 i = 1, end = s.decisionLevel(); i <= end; ++i) { conflict_.push_back(s.decision(i)); }
 			}
@@ -1218,14 +1229,12 @@ bool UncoreMinimize::addCore(Solver& s, const LitPair* lits, uint32 cs, weight_t
 			Core& core = getCore(x);
 			temp_.start(core.bound + 1);
 			for (uint32 k = 0, end = core.size(); k != end; ++k) {
-				Literal x = core.at(k);
-				while (s.value(x.var()) != value_free && s.rootLevel() > eRoot_) {
-					uint32 dl = s.level(x.var());
-					if (dl > eRoot_){ --dl; s.popRootLevel(s.rootLevel() - dl); }
-					else            { s.popRootLevel(s.rootLevel() - eRoot_); }
+				Literal p = core.at(k);
+				while (s.topValue(p.var()) != s.value(p.var()) && s.rootLevel() > eRoot_) {
+					s.popRootLevel(s.rootLevel() - std::max(s.level(p.var())-1, eRoot_));
 					aTop_ = std::min(aTop_, s.rootLevel());
 				}
-				temp_.add(s, x);
+				temp_.add(s, p);
 			}
 			weight_t cw = core.weight;
 			if (!closeCore(s, x, temp_.bound <= 1) || !addCore(s, temp_, cw)) {
@@ -1234,30 +1243,86 @@ bool UncoreMinimize::addCore(Solver& s, const LitPair* lits, uint32 cs, weight_t
 		}
 	}
 	// add new core
-	temp_.start(2);
-	for (uint32 i = 0; i != cs; ++i) { temp_.add(s, lits[i].lit); }
-	if (temp_.relevant()) {
-		return addCore(s, temp_, w);
+	if ((options_ & MinimizeMode_t::usc_clauses) == 0u) {
+		temp_.start(2);
+		for (uint32 i = 0; i != cs; ++i) { temp_.add(s, lits[i].lit); }
+		if (!temp_.unsat()) { 
+			return addCore(s, temp_, w);
+		}
+		Literal fix = !temp_.lits.empty() ? temp_.lits[0].first : lits[0].lit;
+		return temp_.bound < 2 || fixLit(s, fix);
 	}
-	return temp_.lits.empty() || fixLit(s, temp_.lits[0].first);
+	return addClauses(s, lits, cs, w);
+}
+
+bool UncoreMinimize::addClauses(Solver& s, const LitPair* lits, uint32 size, weight_t w) {
+	if (size == 1) { 
+		return fixLit(s, lits[0].lit); 
+	}
+	uint32 i   = size - 1;
+	Literal bp = lits[i].lit; 
+	while (--i != 0) {
+		Literal an = lits[i].lit;
+		Literal bn = posLit(s.pushAuxVar());
+		Literal cn = posLit(s.pushAuxVar());
+		auxAdd_ += 2;
+		addLit(cn, w);
+		if (!add(comp_disj, s, bn, an, bp)) { return false; }
+		if (!add(comp_conj, s, cn, an, bp)) { return false; }
+		bp = bn;
+	}
+	Literal an = lits[i].lit;
+	Literal cn = posLit(s.pushAuxVar());
+	++auxAdd_;
+	addLit(cn, w);
+	return add(comp_conj, s, cn, an, bp);
+}
+bool UncoreMinimize::add(CompType c, Solver& s, Literal head, Literal body1, Literal body2) {
+	typedef ClauseCreator::Result Result;
+	const uint32 flags = ClauseCreator::clause_explicit | ClauseCreator::clause_not_root_sat | ClauseCreator::clause_no_add;
+	const bool    sign = c == comp_conj;
+	uint32       first = 0, last = 3;
+	if ((options_ & MinimizeMode_t::usc_imp_only) != 0u) {
+		first = c == comp_disj;
+		last  = first + (1 + (c == comp_disj));
+	}
+	Literal temp[3][3] = {
+		{ (~head)^ sign,   body1 ^ sign, body2 ^ sign },
+		{   head ^ sign, (~body1)^ sign, negLit(0) }, 
+		{   head ^ sign, (~body2)^ sign, negLit(0) }
+	};
+	for (uint32 i = first, sz = 3; i != last; ++i) {
+		Result res = ClauseCreator::create(s, ClauseRep::create(temp[i], sz, Constraint_t::learnt_other), flags);
+		if (res.local) { closed_.push_back(res.local); }
+		if (!res.ok()) { return false; }
+		sz = 2;
+	}
+	return true;
 }
 bool UncoreMinimize::addCore(Solver& s, const WCTemp& wc, weight_t weight) {
 	typedef WeightConstraint::CPair ResPair;
-	if (!wc.relevant()) {
-		return true;
+	weight_t B = wc.bound;
+	if (B <= 0) { 
+		// constraint is sat and hence conflicting w.r.t new assumption -
+		// relax core
+		lower_ += ((1-B)*weight);
+		B       = 1;
+	}
+	if (static_cast<uint32>(B) > static_cast<uint32>(wc.lits.size())) {
+		// constraint is unsat and hence the new assumption is trivially satisfied
+		return true; 
 	}
 	// create new var for this core
 	if (auxInit_ == UINT32_MAX) { auxInit_ = s.numAuxVars(); }
 	Var newAux = s.pushAuxVar();
 	++auxAdd_;
 	LitData& x = addLit(negLit(newAux), weight);
-	bool         more = static_cast<weight_t>(wc.lits.size()) > wc.bound;
-	WeightLitsRep rep = {&const_cast<WCTemp&>(wc).lits[0], (uint32)wc.lits.size(), std::max(weight_t(0), wc.bound), (weight_t)wc.lits.size()};
+	WeightLitsRep rep = {&const_cast<WCTemp&>(wc).lits[0], (uint32)wc.lits.size(), B, (weight_t)wc.lits.size()};
 	uint32       fset = WeightConstraint::create_explicit | WeightConstraint::create_no_add | WeightConstraint::create_no_freeze | WeightConstraint::create_no_share;
-	if (more) {  fset|= WeightConstraint::create_sat; }
+	if ((options_ & MinimizeMode_t::usc_imp_only) != 0u) { fset |= WeightConstraint::create_only_bfb; }
 	ResPair       res = WeightConstraint::create(s, negLit(newAux), rep, fset);
 	if (res.ok() && res.first()) {
-		x.coreId = allocCore(res.first(), wc.bound, weight, more);
+		x.coreId = allocCore(res.first(), B, weight, rep.bound != rep.reach);
 	}
 	return !s.hasConflict();
 }
@@ -1275,15 +1340,13 @@ uint32 UncoreMinimize::initRoot(Solver& s) {
 // Assigns p at the solver's initial root level.
 bool UncoreMinimize::fixLit(Solver& s, Literal p) {
 	assert(s.decisionLevel() >= eRoot_);
-	if (!s.isTrue(p) || s.level(p.var()) != 0) {
-		if (s.decisionLevel() > eRoot_) {
-			s.popRootLevel(s.rootLevel() - eRoot_);
-			aTop_ = s.rootLevel();
-		}
-		if (eRoot_) { fix_.push_back(p); }
-		return !s.hasConflict() && s.force(p, eRoot_, this);
+	if (s.decisionLevel() > eRoot_ && (!s.isTrue(p) || s.level(p.var()) > eRoot_)) {
+		// go back to root level
+		s.popRootLevel(s.rootLevel() - eRoot_);
+		aTop_ = s.rootLevel();
 	}
-	return !s.hasConflict();
+	if (eRoot_ && s.topValue(p.var()) != trueValue(p)) { fix_.push_back(p); } 
+	return !s.hasConflict() && s.force(p, this);
 }
 // Fixes any remaining assumptions of the active optimization level.
 bool UncoreMinimize::fixLevel(Solver& s) {
@@ -1340,5 +1403,5 @@ void    UncoreMinimize::WCTemp::add(Solver& s, Literal p) {
 	if      (s.topValue(p.var()) == value_free) { lits.push_back(WeightLiteral(p, 1)); }
 	else if (s.isTrue(p))                       { --bound; }
 }
-		
+
 } // end namespaces Clasp
