@@ -21,7 +21,6 @@
 #include "gringo/input/programbuilder.hh"
 #include "gringo/input/literals.hh"
 #include "gringo/input/aggregates.hh"
-#include "gringo/input/statements.hh"
 #include "gringo/input/program.hh"
 #include "gringo/output/output.hh"
 #include "gringo/scripts.hh"
@@ -313,7 +312,7 @@ void NongroundProgramBuilder::rule(Location const &loc, HdLitUid head) {
 }
 
 void NongroundProgramBuilder::rule(Location const &loc, HdLitUid head, BdLitVecUid body) {
-    prg_.add(make_locatable<Rule>(loc, heads_.erase(head), bodies_.erase(body), false));
+    prg_.add(make_locatable<Statement>(loc, heads_.erase(head), bodies_.erase(body), false));
 }
 
 void NongroundProgramBuilder::define(Location const &loc, FWString name, TermUid value, bool defaultDef) {
@@ -321,7 +320,7 @@ void NongroundProgramBuilder::define(Location const &loc, FWString name, TermUid
 }
 
 void NongroundProgramBuilder::optimize(Location const &loc, TermUid weight, TermUid priority, TermVecUid cond, BdLitVecUid body) {
-    prg_.add(make_locatable<WeakConstraint>(loc, terms_.erase(weight), terms_.erase(priority), termvecs_.erase(cond), bodies_.erase(body)));
+    prg_.add(make_locatable<Statement>(loc, terms_.erase(weight), terms_.erase(priority), termvecs_.erase(cond), bodies_.erase(body)));
 }
 
 void NongroundProgramBuilder::showsig(Location const &loc, FWString name, unsigned arity, bool csp) {
@@ -345,7 +344,7 @@ void NongroundProgramBuilder::block(Location const &loc, FWString name, IdVecUid
 }
 
 void NongroundProgramBuilder::external(Location const &loc, LitUid head, BdLitVecUid body) {
-    prg_.add(make_locatable<Rule>(loc, heads_.erase(headlit(head)), bodies_.erase(body), true));
+    prg_.add(make_locatable<Statement>(loc, heads_.erase(headlit(head)), bodies_.erase(body), true));
 }
 
 // }}}
