@@ -342,14 +342,14 @@ ULit BodyAggregate::toLparse(LparseTranslator &x) {
                         SAuxAtom aggr(std::make_shared<AuxAtom>(x.auxAtom()));
                         WeightRule wr(aggr, std::max(0, toInt(y.left) + shift), get_clone(elemPos));
                         wr.toLparse(x);
-                        aggrLits.emplace_back(make_unique<AuxLiteral>(aggr, naf == NAF::NOTNOT));
+                        aggrLits.emplace_back(make_unique<AuxLiteral>(aggr, false));
                     }
                     if (y.right < rng.right) {
                         // aggr :- U+1 { }.
                         SAuxAtom aggr(std::make_shared<AuxAtom>(x.auxAtom()));
                         WeightRule wr(aggr, std::max(0, toInt(y.right) + 1 + shift), get_clone(elemPos));
                         wr.toLparse(x);
-                        aggrLits.emplace_back(make_unique<AuxLiteral>(aggr, naf != NAF::NOTNOT));
+                        aggrLits.emplace_back(make_unique<AuxLiteral>(aggr, true));
                     }
                     LparseRule(auxAtom, std::move(aggrLits)).toLparse(x);
                 }
@@ -365,7 +365,7 @@ ULit BodyAggregate::toLparse(LparseTranslator &x) {
                             // aggr :- 1 { between }
                             if (y.contains(z.second)) { LparseRule(aggr, get_clone(z.first), nullptr).toLparse(x); }
                         }
-                        aggrLits.emplace_back(make_unique<AuxLiteral>(aggr, naf == NAF::NOTNOT));
+                        aggrLits.emplace_back(make_unique<AuxLiteral>(aggr, false));
                     }
                     if (fun == AggregateFunction::MIN ? rng.left < y.left : y.right < rng.right) {
                         SAuxAtom aggr(std::make_shared<AuxAtom>(x.auxAtom()));
@@ -376,7 +376,7 @@ ULit BodyAggregate::toLparse(LparseTranslator &x) {
                                 LparseRule(aggr, get_clone(z.first), nullptr).toLparse(x);
                             }
                         }
-                        aggrLits.emplace_back(make_unique<AuxLiteral>(aggr, naf != NAF::NOTNOT));
+                        aggrLits.emplace_back(make_unique<AuxLiteral>(aggr, true));
                     }
                     LparseRule(auxAtom, std::move(aggrLits)).toLparse(x);
                 }
@@ -384,6 +384,11 @@ ULit BodyAggregate::toLparse(LparseTranslator &x) {
             }
         }
     }
+    if (naf == NAF::NOTNOT) {
+        SAuxAtom negAuxAtom(std::make_shared<AuxAtom>(x.auxAtom()));
+        auxAtom.swap(negAuxAtom);
+        LparseRule(auxAtom, make_unique<AuxLiteral>(negAuxAtom, true), nullptr).toLparse(x);
+    }
     return make_unique<AuxLiteral>(auxAtom, naf != NAF::POS);
 }
 void BodyAggregate::makeEqual(ULit &&, LparseTranslator &) const {
@@ -423,6 +428,8 @@ void AssignmentAggregate::printPlain(std::ostream &out) const {
 ULit AssignmentAggregate::toLparse(LparseTranslator &x) {
     // TODO: this algorithm suffers from severe uglyness!!!
     // TODO: check if false
+    // TODO: the rewriting could be made structure aware
+    //       to exploit common sub-formulas for aggregates that assign multiple value
     SAuxAtom auxAtom(std::make_shared<AuxAtom>(x.auxAtom()));
     using ULitValVec = std::vector<std::pair<ULit,Value>>;
     ULitValVec elemVec;
@@ -847,6 +854,13 @@ void CSPLiteral::makeEqual(ULit &&, LparseTranslator &) const {
 int CSPLiteral::lparseUid(LparseOutputter &) const {
     throw std::logic_error("CSPLiteral::lparseUid: must not be called");
 }
+bool CSPLiteral::invertible() const {
+    return true;
+}
+void CSPLiteral::invert() {
+    std::get<0>(ground) = neg(std::get<0>(ground));
+}
+
 CSPLiteral::~CSPLiteral() { }
 
 // }}}
