@@ -267,7 +267,7 @@ struct DefaultLparseTranslator : LparseTranslator {
         // show explicitely shown csp variables (if not shown above)
         auto it(domains.find(Signature("#show", 2)));
         if (it != domains.end()) {
-            for (auto jt = it->second.exports.begin() + it->second.exports.incOffset, je = it->second.exports.end(); jt != je; ++jt) {
+            for (auto jt = it->second.exports.begin() + it->second.exports.showOffset, je = it->second.exports.end(); jt != je; ++jt) {
                 if ((jt->get().first.args().begin()+1)->num() == 1) {
                     Value val = *(jt->get().first.args().begin());
                     auto bound = boundMap.find(val);
@@ -281,6 +281,7 @@ struct DefaultLparseTranslator : LparseTranslator {
                     }
                 }
             }
+            // Note: showNext is called in LparseHandler::finish
         }
         // check for signatures that did not occur in the program
         for (auto &x : outPreds) {
@@ -607,7 +608,7 @@ struct LparseHandler : StmHandler {
         x.toLparse(trans);
     }
     virtual void incremental() { out.incremental(); }
-    virtual void operator()(PredicateDomain::element_type &head, ExternalType type) {
+    virtual void operator()(PredicateDomain::element_type &head, TruthValue type) {
         if (!head.second.hasUid()) { head.second.uid(out.newUid()); }
         out.printExternal(head.second.uid(), type);
     }
@@ -615,39 +616,44 @@ struct LparseHandler : StmHandler {
         out.disposeMinimize() = trans.minimizeChanged();
         trans.translate(); 
         trans.outputSymbols(out, domains, outPreds);
-        if (!outPreds.empty()) { 
+        if (!outPreds.empty()) {
+            // show what was requested
             for (auto &x : outPreds) {
                 if (!std::get<2>(x)) {
                     auto it(domains.find(std::get<1>(x)));
                     if (it != domains.end()) {
-                        for (auto jt = it->second.exports.begin() + it->second.exports.incOffset, je = it->second.exports.end(); jt != je; ++jt) {
+                        for (auto jt = it->second.exports.begin() + it->second.exports.showOffset, je = it->second.exports.end(); jt != je; ++jt) {
                             if (!jt->get().second.hasUid()) { jt->get().second.uid(out.newUid()); }
                             out.printSymbol(jt->get().second.uid(), jt->get().first);
                         }
+                        it->second.exports.showNext();
                     }
                 }
             }
         }
         else {
+            // show everything non-internal
             for (auto &x : domains) {
                 std::string const &name(*(*x.first).name());
-                if ((name.empty() || name.front() != '#') && outPreds.empty()) {
-                    for (auto jt = x.second.exports.begin() + x.second.exports.incOffset, je = x.second.exports.end(); jt != je; ++jt) {
+                if ((name.empty() || name.front() != '#')) {
+                    for (auto jt = x.second.exports.begin() + x.second.exports.showOffset, je = x.second.exports.end(); jt != je; ++jt) {
                         if (!jt->get().second.hasUid()) { jt->get().second.uid(out.newUid()); }
                         out.printSymbol(jt->get().second.uid(), jt->get().first);
                     }
+                    x.second.exports.showNext();
                 }
             }
         }
-        // Note: show domains are cleared anyway
+        // show terms
         auto it(domains.find(Signature("#show", 2)));
         if (it != domains.end()) {
-            for (auto jt = it->second.exports.begin() + it->second.exports.incOffset, je = it->second.exports.end(); jt != je; ++jt) {
+            for (auto jt = it->second.exports.begin() + it->second.exports.showOffset, je = it->second.exports.end(); jt != je; ++jt) {
                 if ((jt->get().first.args().begin()+1)->num() == 0) {
                     if (!jt->get().second.hasUid()) { jt->get().second.uid(out.newUid()); }
                     out.printSymbol(jt->get().second.uid(), *jt->get().first.args().begin());
                 }
             }
+            it->second.exports.showNext();
         }
         out.finishSymbols();
     }
@@ -656,7 +662,7 @@ struct LparseHandler : StmHandler {
             for (auto &x : domains) {
                 Signature sig = *x.first;
                 std::string const &name = *sig.name();
-                if ((atomset & Model::ATOMS && !name.empty() && name.front() != '#') || (atomset & Model::SHOWN && showSig(outPreds, *x.first))) {
+                if (((atomset & Model::ATOMS || (atomset & Model::SHOWN && showSig(outPreds, *x.first))) && !name.empty() && name.front() != '#')) {
                     for (auto &y: x.second.domain) {
                         if (y.second.defined() && y.second.hasUid() && isTrue(y.second.uid())) { atoms.emplace_back(y.first); }
                     }
@@ -688,11 +694,12 @@ struct LparsePlainHandler : StmHandler {
         : trans([&out](Statement const &x)                { x.printPlain(out); })
         , out(out) { }
     virtual void operator()(Statement &x)                 { x.toLparse(trans); }
-    virtual void operator()(PredicateDomain::element_type &head, ExternalType type) {
+    virtual void operator()(PredicateDomain::element_type &head, TruthValue type) {
         switch (type) {
-            case ExternalType::E_FALSE: { out << "#external " << head.first << ".\n"; break; }
-            case ExternalType::E_TRUE:  { out << "#external " << head.first << "=true.\n"; break; }
-            case ExternalType::E_FREE:  { out << "#external " << head.first << "=free.\n"; break; }
+            case TruthValue::False: { out << "#external " << head.first << ".\n"; break; }
+            case TruthValue::True:  { out << "#external " << head.first << "=true.\n"; break; }
+            case TruthValue::Free:  { out << "#external " << head.first << "=free.\n"; break; }
+            case TruthValue::Open:  { out << "#external " << head.first << "=open.\n"; break; }
         }
     }
     virtual void finish(PredDomMap &, OutputPredicates &) { trans.translate(); }
@@ -710,11 +717,12 @@ struct PlainHandler : StmHandler {
     PlainHandler(std::ostream &out) : out(out)            { }
     virtual ~PlainHandler()                               { }
     virtual void operator()(Statement &x)                 { x.printPlain(out); }
-    virtual void operator()(PredicateDomain::element_type &head, ExternalType type) {
+    virtual void operator()(PredicateDomain::element_type &head, TruthValue type) {
         switch (type) {
-            case ExternalType::E_FALSE: { out << "#external " << head.first << ".\n"; break; }
-            case ExternalType::E_TRUE:  { out << "#external " << head.first << "=true.\n"; break; }
-            case ExternalType::E_FREE:  { out << "#external " << head.first << "=free.\n"; break; }
+            case TruthValue::False: { out << "#external " << head.first << ".\n"; break; }
+            case TruthValue::True:  { out << "#external " << head.first << "=true.\n"; break; }
+            case TruthValue::Free:  { out << "#external " << head.first << "=free.\n"; break; }
+            case TruthValue::Open:  { out << "#external " << head.first << "=open.\n"; break; }
         }
     }
     virtual void finish(PredDomMap &, OutputPredicates &outPreds) { 
@@ -799,11 +807,12 @@ void PlainLparseOutputter::printDisjunctiveRule(AtomVec const &head, LitVec cons
     for (auto &x : body) { if (x > 0) { out << " " << +x; } }
     out << "\n";
 }
-void PlainLparseOutputter::printExternal(unsigned atomUid, ExternalType type) { 
+void PlainLparseOutputter::printExternal(unsigned atomUid, TruthValue type) { 
     switch (type) {
-        case ExternalType::E_FALSE: { out << "91 " << atomUid << " 0\n"; break; }
-        case ExternalType::E_TRUE:  { out << "91 " << atomUid << " 1\n"; break; }
-        case ExternalType::E_FREE:  { out << "92 " << atomUid << "\n"; break; }
+        case TruthValue::False: { out << "91 " << atomUid << " 0\n"; break; }
+        case TruthValue::True:  { out << "91 " << atomUid << " 1\n"; break; }
+        case TruthValue::Open:  { out << "91 " << atomUid << " 2\n"; break; }
+        case TruthValue::Free:  { out << "92 " << atomUid << "\n"; break; }
     }
 }
 unsigned PlainLparseOutputter::falseUid()                                   { return 1; }
@@ -826,7 +835,7 @@ void OutputBase::output(Value const &val) {
     auto it(domains.find(val.sig()));
     assert(it != domains.end());
     auto ret(it->second.insert(val, true));
-    if (!std::get<2>(ret) || !std::get<1>(ret)) {
+    if (!std::get<2>(ret)) {
         tempRule.head = std::get<0>(ret);
         tempRule.body.clear();
         (*handler)(tempRule);
@@ -835,7 +844,7 @@ void OutputBase::output(Value const &val) {
 void OutputBase::incremental() {
     handler->incremental();
 }
-void OutputBase::external(PredicateDomain::element_type &head, ExternalType type) {
+void OutputBase::external(PredicateDomain::element_type &head, TruthValue type) {
     (*handler)(head, type);
 }
 void OutputBase::output(UStm &&x) {
