@@ -32,8 +32,10 @@ class TestLparse : public CppUnit::TestFixture {
         CPPUNIT_TEST(test_projectionBug);
         CPPUNIT_TEST(test_recAntiAggr);
         CPPUNIT_TEST(test_aggregateBug);
+        CPPUNIT_TEST(test_aggregateNotNot);
         CPPUNIT_TEST(test_aggregateRecBug);
         CPPUNIT_TEST(test_headAggregateBug);
+        CPPUNIT_TEST(test_headAggrPropagateBug);
         CPPUNIT_TEST(test_symTabBug);
         CPPUNIT_TEST(test_mutexBug);
         CPPUNIT_TEST(test_head);
@@ -60,8 +62,10 @@ public:
     void test_projectionBug();
     void test_recAntiAggr();
     void test_aggregateBug();
+    void test_aggregateNotNot();
     void test_aggregateRecBug();
     void test_headAggregateBug();
+    void test_headAggrPropagateBug();
     void test_symTabBug();
     void test_mutexBug();
     void test_head();
@@ -110,6 +114,29 @@ void TestLparse::test_aggregateBug() {
         "b(1)."
         "{ b(X) } :- a(X).")));
 }
+void TestLparse::test_aggregateNotNot() {
+    CPPUNIT_ASSERT_EQUAL(
+        S("[[p(2)],[p(2),p(3)],[p(2),p(3),p(4)],[p(2),p(3),p(4),p(5)],[p(2),p(3),p(5)],[p(2),p(4)],[p(2),p(4),p(5)],[p(2),p(5)],[p(4)],[p(4),p(5)]]"),
+        IO::to_string(solve(
+        "{ p(1..5) }."
+        ":- not not 2 != #min { X:p(X) } != 4.")));
+    CPPUNIT_ASSERT_EQUAL(
+        S("[[p(2)],[p(2),p(3)],[p(2),p(3),p(4)],[p(2),p(3),p(4),p(5)],[p(2),p(3),p(5)],[p(2),p(4)],[p(2),p(4),p(5)],[p(2),p(5)],[p(4)],[p(4),p(5)]]"),
+        IO::to_string(solve(
+        "{ p(1..5) }."
+        ":- 2 != #min { X:p(X) } != 4.")));
+    CPPUNIT_ASSERT_EQUAL(
+        S("[[p(2)],[p(2),p(3)],[p(2),p(3),p(4)],[p(2),p(3),p(4),p(5)],[p(2),p(3),p(5)],[p(2),p(4)],[p(2),p(4),p(5)],[p(2),p(5)],[p(4)],[p(4),p(5)]]"),
+        IO::to_string(solve(
+        "{ p(1..5) }."
+        "h :- not 2 != #min { X:p(X) } != 4."
+        ":- not h.", {"p("})));
+    CPPUNIT_ASSERT_EQUAL(
+        S("[[p(2)],[p(2),p(3)],[p(2),p(3),p(4)],[p(2),p(3),p(4),p(5)],[p(2),p(3),p(5)],[p(2),p(4)],[p(2),p(4),p(5)],[p(2),p(5)],[p(4)],[p(4),p(5)]]"),
+        IO::to_string(solve(
+        "{ p(1..5) }."
+        ":- not 2 = #min { X:p(X) }, not #min { X:p(X) } = 4.")));
+}
 void TestLparse::test_aggregateRecBug() {
     Gringo::Test::Messages msg;
     CPPUNIT_ASSERT_EQUAL(
@@ -166,6 +193,23 @@ void TestLparse::test_recAntiAggr() {
         S("[[],[p]]"),
         IO::to_string(solve("p :- not not #sum { 1:not p } < 1.")));
 }
+void TestLparse::test_headAggrPropagateBug() {
+    Gringo::Test::Messages msg;
+    CPPUNIT_ASSERT_EQUAL(
+        S(
+            "[[a(c),a(p),b(c,d),b(c,e),b(c,f),b(c,g),b(p,d),b(p,e),b(p,f),b(p,g)],"
+            "[a(c),a(p),b(c,d),b(c,e),b(c,g),b(p,d),b(p,e),b(p,g),e(3)],"
+            "[b(c,d),b(c,e),b(p,d),b(p,e),e(2),e(3)]]"), 
+        IO::to_string(solve(
+            "b(S,h) :- b(S,X), c.\n"
+            "b(c,d).\n"
+            "b(p,X) :- b(c,X).\n"
+            "b(c,e).\n"
+            "a(S)   :- b(S,g).\n"
+            "1 = { e(3); b(S,f) : a(S) } :- b(S,e).\n"
+            "1 = { e(2); b(S,g) } :- b(S,d).\n")));
+    CPPUNIT_ASSERT_EQUAL(S("[-:1:19-20: warning: atom is undefined:\n  c\n]"), IO::to_string(msg.messages));
+}
 void TestLparse::test_mutexBug() {
 #ifdef WITH_LUA
     CPPUNIT_ASSERT_EQUAL(
